# -*- coding: utf-8 -*-
"""
Created on Wed Oct 16 11:51:21 2024

@author: nicon y corir (mentira no hice nada)
"""


import numpy as np
import matplotlib.pyplot as plt
from scipy import signal

# Variables globales para almacenar el último centro
last_xlim = None
last_ylim = None

def Grating(k, l, size, Binary=True, Blaze=False, Inverted=False, powerblaze=1, BinaryFactor=0.25, Nx=10000, Ny = 5000, phase1storder=0, mini=0, maxi=255, x0=0, y0=0):
    
    x = np.linspace(-size, size, Nx)
    y = np.linspace(-size, size, Ny)
    
    if Binary:
        if not Inverted:
            Pattern = [[maxi if 0.5*(1 - np.cos(k*xi - l*np.arctan2(xi, yj) - phase1storder)) > BinaryFactor else mini for xi in x] for yj in y]
        else:
            Pattern = [[maxi if 0.5*(1 - np.cos(k*xi - l*np.arctan2(xi, yj) - phase1storder)) > BinaryFactor else mini for yj in y] for xi in x]

    else:
        if Blaze:
            Pattern = []
            for yj in y:
                Pat=[]
                for xi in x:
                    grati = maxi*0.5*(1 - arbpower_sawtooth(k*(xi-x0) - l*np.arctan2(xi-x0, yj-y0) - phase1storder, power=powerblaze, width = 1))
                    Pat.append(grati)
                Pattern.append(Pat)
            
        else:
            Pattern = []
            for yj in y:
                Pat=[]
                for xi in x:
                    Pat.append(0.5*(1 - np.cos(k*(xi-x0) - l*np.arctan2(xi-x0, yj-y0) - phase1storder)))
                Pattern.append(Pat)

    return Pattern, x, y


def arbpower_sawtooth(t, power, width=1):
    """
    Función para armar patrones de difracción con blaze
        -power: parámetro de potencia. debe ser positivo. si está entre 0 y 1 es una función raiz. si es 1, es lineal. si es más de 1, potencia.
        -width: si es 1 tiene una orientación y si es 0 tiene otra. no poner valores intermedios (por ahora)
    """
    # Normaliza el valor de t a un rango de 0 a 1
    t_mod = np.mod(t, 2 * np.pi) / (2 * np.pi)
    
    # Para la parte creciente del ciclo: función de raíz cuadrada
    positive_part = t_mod**power
    
    # Para la parte decreciente del ciclo: invertimos la raíz cuadrada
    negative_part = -(1 - t_mod)**power
    
    # Combina ambas partes
    result =2*np.abs(np.where(t_mod < width, positive_part, negative_part))-1
    
    return result

def on_key(event):
    global last_xlim, last_ylim  # Para acceder a las variables globales
    xlim = ax.get_xlim()
    ylim = ax.get_ylim()
    
    step_sizex = (xlim[1] - xlim[0]) / len(y)  # Ajusta este valor para cambiar la velocidad de desplazamiento
    step_sizey = (ylim[1] - ylim[0]) / len(x)

    if event.key == "right":
        ax.set_xlim(xlim[0] - step_sizey, xlim[1] - step_sizey)  # Mover derecha
    elif event.key == "left":
        ax.set_xlim(xlim[0] + step_sizey, xlim[1] + step_sizey)  # Mover izquierda
    elif event.key == "up":
        ax.set_ylim(ylim[0] - step_sizex, ylim[1] - step_sizex)  # Mover arriba
    elif event.key == "down":
        ax.set_ylim(ylim[0] + step_sizex, ylim[1] + step_sizex)  # Mover abajo

    # Guardar el nuevo centro
    last_xlim = ax.get_xlim()
    last_ylim = ax.get_ylim()

    fig.canvas.draw_idle()  # Redibujar la imagen sin regenerarla

def on_close(event):
    global last_xlim, last_ylim
    # Guardar los límites actuales cuando se cierre la ventana
    last_xlim = ax.get_xlim()
    last_ylim = ax.get_ylim()
    print(f'Última posición guardada: xlim={last_xlim}, ylim={last_ylim}')

def display_image():
    global fig, ax

    fig, ax = plt.subplots(figsize=(8, 6), dpi=100)
    c = plt.pcolor(Pattern, cmap='gray', vmin=0, vmax=255)

    plt.xticks([])
    plt.yticks([])
    plt.gca().set_position([0, 0, 1, 1])

    manager = plt.get_current_fig_manager()
    manager.full_screen_toggle()
    fig.canvas.window().statusBar().setVisible(False)
    manager.toolbar.hide()

    # Restaurar la última posición si existe
    if last_xlim and last_ylim:
        ax.set_xlim(last_xlim)
        ax.set_ylim(last_ylim)

    # Conectar eventos
    fig.canvas.mpl_connect("key_press_event", on_key)
    fig.canvas.mpl_connect("close_event", on_close)

    plt.show()

#%%

"""
Codigo para hacer un tenedor con todo ajustado y andando
"""


size=1 #esto en realidad no cambia absolutamente nada
Nx = 800
Ny = 600

linewidth = 400

k = 1*np.pi/((linewidth/10000)*size)

l = 1 #para usar redes comunes pone l=0

mini, maxi = 0,255 #numeros de 0 a 255 que definen los tonos de gris de las franjas

binary = 'binary'  #posibles: binary o notbinary. para usar blaze usa notbinary

Inverted=True

Blaze=False
bfactor = 2
print("aguante boca")

# Phase = [0,45,90,135,180]

Phase = [0]
for p in Phase:
# p = 90
    phase = p*np.pi/180
    binaryfactor = 0.5
    
    if binary == 'binary':
        Pattern, x, y = Grating(np.pi/((linewidth/10000)*size), l, size, Binary=True, Blaze=Blaze, Inverted=Inverted, powerblaze=bfactor, BinaryFactor=binaryfactor, Nx=Nx, Ny=Ny, phase1storder=phase, mini=mini, maxi=maxi)
    else:
        Pattern, x, y = Grating(np.pi/((linewidth/10000)*size), l, size, Binary=False, Blaze=Blaze, Inverted=Inverted, powerblaze=bfactor, BinaryFactor=binaryfactor, Nx=Nx, Ny=Ny, phase1storder=phase, mini=mini, maxi=maxi)

fig, ax = plt.subplots(figsize=(8, 6), dpi = 100)
c = plt.pcolor(Pattern,cmap='gray', vmin=0,vmax=255)

plt.xticks([])
plt.yticks([])

plt.gca().set_position([0,0,1,1])
#plt.subplots_adjust(left=0, right=1, top=1. bottom=0)
#plt.rcParams["figure.autolayout"] = False
manager = plt.get_current_fig_manager()

manager.full_screen_toggle() #con f o ctrl+f se sale del full screen

fig.canvas.window().statusBar().setVisible(False)

manager.toolbar.hide()

print(f'done {p}')

# Conectar la función al evento de teclado
fig.canvas.mpl_connect("key_press_event", on_key)

# Para llamar a display_image() cuando quieras mostrar la imagen
display_image()


#%%

"""
Codigo para hacer dos tenedores de helicidades l1 y l2
"""

size=1 #esto en realidad no cambia absolutamente nada
Nx = 800
Ny = 300

linewidth = 1200

k = 1*np.pi/((linewidth/10000)*size)

l1, l2 = 1, 1 #para usar redes comunes pone l=0

mini, maxi = 0,255 #numeros de 0 a 255 que definen los tonos de gris de las franjas

binary = 'binary'  #posibles: binary o notbinary. para usar blaze usa notbinary
orientation = 'vertical' #posibles: vertical (tenedores uno arriba del otro) u horizontal (tenedores uno al lado del otro)

Inverted=True

Blaze=False
bfactor = 2
print("aguante boca")

# Phase = [0,45,90,135,180]

Phase = [0]
for p in Phase:
# p = 90
    phase = p*np.pi/180
    binaryfactor = 0.5
    
    if binary == 'binary':
        Pattern1, x1, y1 = Grating(np.pi/((linewidth/10000)*size), l1, size, Binary=True, Blaze=Blaze, Inverted=Inverted, powerblaze=bfactor, BinaryFactor=binaryfactor, Nx=Nx, Ny=Ny, phase1storder=phase, mini=mini, maxi=maxi)
        Pattern2, x2, y2 = Grating(np.pi/((linewidth/10000)*size), l2, size, Binary=True, Blaze=Blaze, Inverted=Inverted, powerblaze=bfactor, BinaryFactor=binaryfactor, Nx=Nx, Ny=Ny, phase1storder=phase, mini=mini, maxi=maxi)
    else:
        Pattern1, x1, y1 = Grating(np.pi/((linewidth/10000)*size), l1, size, Binary=False, Blaze=Blaze, Inverted=Inverted, powerblaze=bfactor, BinaryFactor=binaryfactor, Nx=Nx, Ny=Ny, phase1storder=phase, mini=mini, maxi=maxi)
        Pattern2, x2, y2 = Grating(np.pi/((linewidth/10000)*size), l2, size, Binary=False, Blaze=Blaze, Inverted=Inverted, powerblaze=bfactor, BinaryFactor=binaryfactor, Nx=Nx, Ny=Ny, phase1storder=phase, mini=mini, maxi=maxi)


def on_key2(event):
    xlim1, ylim1 = ax1.get_xlim(), ax1.get_ylim()
    xlim2, ylim2 = ax2.get_xlim(), ax2.get_ylim()    
    step_sizex1 = (max(xlim1) - min(xlim1))/len(y1)  # Ajusta para cambiar la velocidad
    step_sizey1 = (max(ylim1) - min(ylim1))/len(x1)
    step_sizex2 = (max(xlim2) - min(xlim2))/len(y2)  # Ajusta para cambiar la velocidad
    step_sizey2 = (max(ylim2) - min(ylim2))/len(x2)
    # print(step_sizex)
    # print(step_sizey)

    # xlim = ax.get_xlim()
    # ylim = ax.get_ylim()
    # step_sizex = (max(xlim) - min(xlim))/len(x)  # Ajusta este valor para cambiar la velocidad de desplazamiento
    # step_sizey = (max(ylim) - min(ylim))/len(y)


    # Verifica si Shift está presionado para determinar cuál patrón mover
    if event.key == "shift+right":
        xlim2 = (xlim2[0] - step_sizex2, xlim2[1] - step_sizex2)
        ax2.set_xlim(xlim2)
    elif event.key == "shift+left":
        xlim2 = (xlim2[0] + step_sizex2, xlim2[1] + step_sizex2)
        ax2.set_xlim(xlim2)
    elif event.key == "shift+up":
        ylim2 = (ylim2[0] - step_sizey2, ylim2[1] - step_sizey2)
        ax2.set_ylim(ylim2)
    elif event.key == "shift+down":
        ylim2 = (ylim2[0] + step_sizey2, ylim2[1] + step_sizey2)
        ax2.set_ylim(ylim2)
    elif event.key == "right":
        xlim1 = (xlim1[0] - step_sizex1, xlim1[1] - step_sizex1)
        ax1.set_xlim(xlim1)
    elif event.key == "left":
        xlim1 = (xlim1[0] + step_sizex1, xlim1[1] + step_sizex1)
        ax1.set_xlim(xlim1)
    elif event.key == "up":
        ylim1 = (ylim1[0] - step_sizey1, ylim1[1] - step_sizey1)
        ax1.set_ylim(ylim1)
    elif event.key == "down":
        ylim1 = (ylim1[0] + step_sizey1, ylim1[1] + step_sizey1)
        ax1.set_ylim(ylim1)
    fig.canvas.draw_idle()  # Redibujar la imagen sin regenerarla

if orientation=='vertical':
    fig, (ax1, ax2) = plt.subplots(2, 1, figsize=(8, 6), dpi=100)
elif orientation=='horizontal':
    fig, (ax1, ax2) = plt.subplots(1, 2, figsize=(8, 6), dpi=100)
else:
    print('no se entendio la orientacion')
    raise
# Grafica Pattern1 en el primer eje
ax1.pcolor(Pattern1, cmap='gray', vmin=0, vmax=255,edgecolors='none')
ax1.set_xticks([])
ax1.set_yticks([])

for spine in ax1.spines.values():
    spine.set_visible(False)

# Grafica Pattern2 en el segundo eje
ax2.pcolor(Pattern2, cmap='gray', vmin=0, vmax=255,edgecolors='none')
ax2.set_xticks([])
ax2.set_yticks([])

for spine in ax2.spines.values():
    spine.set_visible(False)

# Configuración para pantalla completa y sin bordes
plt.subplots_adjust(left=0, right=1, top=1, bottom=0, wspace=0, hspace=0)
manager = plt.get_current_fig_manager()
manager.full_screen_toggle()
fig.canvas.window().statusBar().setVisible(False)
manager.toolbar.hide()

plt.show()

def on_keytest(event):
    print('you pressed', event.key, event.xdata, event.ydata)

print(f'done {p}')

# Conectar la función al evento de teclado
fig.canvas.mpl_connect("key_press_event", on_key2)
